%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% A Demo for GP regression using a product kernel.  
clear;
close all;

if (strcmp('fleet', getenv('USERNAME')))
    codeRoot = '/h/54/fleet/Research/';
    addpath([codeRoot, 'GPDM/gpdm/src/netlab/']);
    addpath([codeRoot, 'GPDM/gpdm/src/gplvm/']);
    addpath([codeRoot, 'GPDM/gpdm/src/gpdm/']);
    addpath([codeRoot, 'GPDM/gpdm/src/util/']);
    addpath([codeRoot, 'GPDM/gpdm/']);
    addpath([codeRoot, 'GPSC/mgp/']);
    addpath([codeRoot, 'GPSC/mgp/qtfm/']);
    dataDir = [codeRoot, 'GPDM/gpdm/'];
%elseif (strcmp('jmwang', getenv('USERNAME')))
else
    addpath ../gpdm/src/netlab ../gpdm/src/gpdm ../gpdm/src/gplvm ...
        ../gpdm ../gpdm/src/util ./lbfgs ./LHSutils
end


global L_BFGS OPT_MASK

format long;

set(0,'DefaultFigureRenderer','opengl')

% Define optimizer to use
L_BFGS = 0;

% form of kernels
USE_TWO_RBFS = 1;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  Define a simple test case for the regression
%   where
%           Z = 10 * sin(A) * (B-10) + 20 + noise; 
%   as a function of factors A and B
%   Below we have a scalar function case.  it can also be generalized
%   to the multivariate case.
%   Also, this is the pure regression case where all inputs are known.
%      it is also possible to generalize to cases where "content" inputs
%      are known but we still want to optimize some style factors

load nVDData
load ../3d_control/maskrestrictlowa11s
Y0 = Y; 
indices = find(maskrestrictlowa11s == 1); 
Y = Y(:,indices); 

% regression output
N = size(Y,1);   % number of training data samples
D = size(Y,2);   % dimension of the output

meanData = mean(Y);
Y = Y - repmat(meanData, N, 1);
vY = var(Y); 
for d = 1:D
    Y(:,d) = Y(:,d)/sqrt(vY(d)); 
end

% [v, u] = pca(Y);
% v(find(v<0))=0;
% XX = Y*u(:, 1:30)*diag(1./sqrt(v(1:30)));
% YY = XX*diag(sqrt(v(1:30)))*u(:, 1:30)'; 
% 
% for d = 1:D
%     YY(:,d) = YY(:,d)*sqrt(vY(d)); 
% end
% YY = YY + repmat(meanData, N, 1);
% ZZ = Y0; 
% ZZ(:,indices) = YY;  


% regression input
%   (Append 1 to every input to account for translations)
Us{1} = [X(:,1) ones(N,1)];  
Us{2} = [X(:,2) ones(N,1)]; 

% hyperparameters for the regression kernels
Hps{1} = 1; 		% Use RBF kernel for factor A
if (USE_TWO_RBFS == 1)
    Hps{2} = 1;
else
    Hps{2} = []; % Use Linear kernel for factor B
end
theta = 1; 		    % Initial overall kernel scale
beta = exp(1); 		% Initial output noise


% These should stay the same for all regression problems
% (they're used for the hyperparameters of time series models)
Phps{1} = [];
Phps{2} = []; 

% w is a scalar (variance) for each output dim 
w = ones(1,D);

% also used mainly for time series data
segments{1} = [1]; 
segments{2} = [1]; 

% indicates which factors are used to fit which data
factor_idx = [[1:N]' [1:N]']; 

% Ns: number of data points
% qs: dimension of latent factor
% Nhps: number of kernel hyperparameters for factor
% Nphps: number of kernel hyperparameters for dynical prior (if needed)
% priorModel: type of kernel for dynamical prior (if needed)
% periodic: for dynamical model (s needed)
% covS: 
% meanS: 
% G: spline control points (not used now)
% BalanceTerm: 
MGPopts{1} = struct('Ns', N, 'qs', size(Us{1},2), 'Nhps', size(Hps{1},2), 'Nphps', 0, 'priorModel', [], ...
        'periodic', 0, 'covS', [], 'meanS', [], 'G', [], 'BalanceTerm', 1); 
MGPopts{2} = struct('Ns', N, 'qs', size(Us{2},2), 'Nhps', size(Hps{2},2), 'Nphps', 0, 'priorModel', [], ...
        'periodic', 0, 'covS', [], 'meanS', [], 'G', [], 'BalanceTerm', 1); 


% Collect all inputs and unknowns to the model
params = toOptimizerParams(Us, Hps, Phps, theta, beta); 

% which latent factors (inputs) should be optimized
%      fixed = 0, optimize = 1;
optInputs = [0 0];

% Create mask for optimizer to specify which variables to optimize
OPT_MASK = createOptimizerMask(params, MGPopts, optInputs);
optIdx = setdiff(1:length(params),OPT_MASK); 
realparams = params(optIdx); 

fn = @neglogprob_of_everything;

inItr = 1000; 
if L_BFGS == 1
    opts = lbfgs_options('iprint', 1, 'maxits', inItr, 'cb', @test_callback);
else
    opts = optimset('Display','iter','GradObj','on','MaxIter', inItr, 'LargeScale', 'off', 'DerivativeCheck', 'off');
end

if L_BFGS == 1
    [realparams,fx,exitflag,userdata] = lbfgs(@(x)fn(x, params, Y, w, factor_idx, segments, MGPopts),realparams,opts);
    realparams = realparams';
else
    [realparams,fx,exitflag] = fminunc(@(x)fn(x, params, Y, w, factor_idx, segments, MGPopts),realparams,opts);
end

params(optIdx) = realparams; 
[Us, Hps, Phps, theta, beta] = fromOptimizerParams(params, MGPopts); 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Make predictions on test data
[K, invK] = computeNModeKernel(Us, Hps, factor_idx, theta, beta);
a_star = [0.1:0.025:0.8];
b_star = [0.3:0.025:1.0];
[As Bs] = meshgrid(a_star, b_star); 
testPts = [As(:) Bs(:)]; 
for k = 1:size(testPts,1) 
    Us_star{1} = [testPts(k,1) 1]; 
    Us_star{2} = [testPts(k,2) 1];
    [Ysample(k,:) Ymean(k,:) Yvar(k)] = ...
        sampleNModeReconstruction(Us_star, Us, Y, factor_idx, Hps, theta, beta, w, K, invK); 
end

for d = 1:D
    Ymean(:,d) = Ymean(:,d)*sqrt(vY(d)); 
end
Ymean = Ymean + repmat(meanData, size(testPts,1), 1);



idx = 197; 
Z = Y0(1,:)'; 
testPts(idx,:)
Z(indices) = Ymean(idx,:)'; 

save ../3d_control/interpctrl Z -ascii

% hold on;
% scatter3(As(:), Bs(:), Yvar, 'filled');

% Z = Y0; 
% Z(1,indices) = Ymean;
% Z1 = Z(1,:)'; 
% 
% save ../3d_control/interpctrl Z1 -ascii

% Zs = 10*sin(As).*(Bs-10)+20; 
% Zs = Zs + 5*randn(size(Zs));
% Yorig = Zs(:);
% 
% hold on;
% scatter3(A(:), B(:), Y, 'filled');
% %scatter3(As(:), Bs(:), Yorig, 'filled'); 
% Zt = reshape(Ymean, size(As,1), size(As,2)); 
% Ztv = reshape(Yvar, size(As,1), size(As,2)); 
% scatter3(As(:), Bs(:), Zt(:));
% scatter3([As(:); As(:)], [Bs(:); Bs(:)], [Zt(:)+sqrt(Ztv(:)); Zt(:)-sqrt(Ztv(:))]);
% legend('Training Data', 'Ground Truth', 'Mean Prediction', 'One STD');
