function [Y, initY, segments] = loadMocapData(files, startFrames, ... 
    skipFrames, endFrames, samples)

addpath qtfm

if (~exist('samples','var'))
    samples = 0;
end

dims =[6 3 3 3 3 3 3 2 3 1 1 2 1 2 2 3 1 1 2 1 2 3 1 2 1 3 1 2 1];
locations = [1 7 10 13 16 19 22 25 27 30 31 32 34 35 37 39 42 43 44 46 47 49 52 53 55 56 59 60 62];
mask = [25:26 32:33 34 35:38 44:45 46 47:48 55 62];


N = size(files,2);
Y = [];
curY = {};
end_segments = [];
for n = 1:N
    curY{n} = amc_to_matrix(cell2mat(files(n)));
    curY{n}(:,4:end) = (curY{n}(:,4:end)/180)*pi;
    
%     theta = curY{n}(:,4:6);
%     Rx = [1 0 0; 0 cos(theta(1)) sin(theta(1)); 0 -sin(theta(1)) cos(theta(1))];
%     Ry = [cos(theta(2)) 0 -sin(theta(2)); 0 1 0; sin(theta(2)) 0 cos(theta(2))];
%     Rz = [cos(theta(3)) sin(theta(3)) 0; -sin(theta(3)) cos(theta(3)) 0; 0 0 1];
%     R = Rz*Ry*Rx;
 
    curY{n} = toExpMap(curY{n}, dims, locations); 

    curY{n}(:,mask) = [];


    Y = [Y; curY{n}];
    if (n == 1)
        end_segments = size(Y,1);
    else
        end_segments = [end_segments end_segments(end) + size(curY{n},1)];
    end
end
segments = end_segments + 1;
segments = [1 segments];
segments(end) = [];

initY = repmat(Y(1,:),length(segments),1);
initY(:,2) = Y(segments,2);
% 
% globY = Y(2:end, 1:3) - Y(1:end-1, 1:3);
% globY(end_segments,1:3) = globY(end_segments-1,1:3);
% 

% Y = [globY Y(:, 4:end)];
% for i = 1:length(segments)-1
%     Y(segments(i):segments(i+1)-1, 1:3)  = ... 
%         Y(segments(i):segments(i+1)-1, 1:3) - repmat(Y(segments(i), 1:3), ... 
%         segments(i+1) - segments(i), 1); 
% end
% idx = segments(end);
% Y(idx:end, 1:3)  = ...
%     Y(idx:end, 1:3) - repmat(Y(idx, 1:3), size(Y,1) + 1 - idx, 1);

numJoints = (size(Y,2) - 3);

Y = [];
for n = 1:N
    if (endFrames(n) > 0)
        curY{n} = curY{n}(startFrames(n):skipFrames(n):endFrames(n), :);
    else
        curY{n} = curY{n}(startFrames(n):skipFrames(n):end, :);
    end
    
    if (samples > 0)
        num = size(curY{n},1);
        X = (samples - 1)/(num - 1)*[1:num]' + (1 - (samples - 1)/(num - 1));
        newY = zeros(samples, size(curY{n},2));
        for d = 1:size(curY{n},2)
            newY(:,d) = interp1(X, curY{n}(:,d), [1:samples]', 'spline', 'extrap');
        end
    else
        newY = curY{n};
    end
    Y = [Y; newY];

    if (n == 1)
        end_segments = size(newY,1);
    else
        end_segments = [end_segments end_segments(end) + size(newY,1)];
    end
end


segments = end_segments + 1;
segments = [1 segments];
segments(end) = [];

% q_init = Exp_to_Quaternion(Y(1,4:6));
% 
% for n = 2:N
%     q_now = Exp_to_Quaternion(Y(segments(n),4:6));
%     q = q_init*inv(q_now);
%     
%     if (n == N)
%         idx = segments(n):size(Y,1); 
%     else
%         idx = segments(n):(segments(n+1)-1);
%     end
%     q_vec = Exp_to_Quaternion(Y(idx,4:6));
%     
%     % Warp global orientation of subsequent segments to the first segment
%     q_vec = q_init.*inv(q_now).*q_vec;
%     
%     expmap = Quaternion_to_Exp(q_vec);
%     Y(idx,4:6) = [x(expmap) y(expmap) z(expmap)]; 
%     
%     v = Y(idx,1:3);
%     q_vec = quaternion(zeros(length(idx),1), v(:,1), v(:,2), v(:,3)); 
%     q_vec = q*q_vec*inv(q); 
%     Y(idx,1:3) = [x(q_vec) y(q_vec) z(q_vec)]; 
% end

% Predict global velocity.
globY = Y(2:end, 1:3) - Y(1:end-1, 1:3);
globY(end_segments,1:3) = globY(end_segments-1,1:3);
velY = Y(2:end, 4:end) - Y(1:end-1, 4:end);
velY(end_segments,4:end) = velY(end_segments-1,4:end);
Y = [globY Y(:, 4:end) velY];

% globY = Y(2:end, 1:6) - Y(1:end-1, 1:6);
% globY(end_segments,1:6) = globY(end_segments-1,1:6);
% velY = Y(2:end, 7:end) - Y(1:end-1, 7:end);
% velY(end_segments,7:end) = velY(end_segments-1,7:end);
% Y = [globY Y(:, 7:end) velY];

% Predict global position.
% for i = 1:length(segments)-1
%     Y(segments(i):segments(i+1)-1, 1:3)  = ... 
%         Y(segments(i):segments(i+1)-1, 1:3) - repmat(Y(segments(i), 1:3), ... 
%         segments(i+1) - segments(i), 1); 
% end
% idx = segments(end);
% Y(idx:end, 1:3)  = ...
%     Y(idx:end, 1:3) - repmat(Y(idx, 1:3), size(Y,1) + 1 - idx, 1);


rmpath qtfm
