load lastMGPModel

MODEL = 1; 
if exist('t', 'var')
    MODEL = 2;
end

% Compute reconstruction and content dynamics kernels
[K, invK] = ...
    computeNModeKernel(Us, Hps, factor_idx, theta, beta);

if (MODEL == 1) 
    [Xin Xout] = priorIO(Us{1}, segments{1}, MGPopts{1}.priorModel);
    [Kd invKd] = computePriorKernel(Xin, Phps{1}, MGPopts{1}.priorModel(3));
end
% For notational convenience
for n = 1:length(MGPopts)
    Ns(n) = MGPopts{n}.Ns; 
    qs(n) = MGPopts{n}.qs; 
end


% Circular content model needs an additional bilinear mapping to predict
% step size from style to synthesize animation
if (MODEL == 2)
    A = [];
    B = [];
    for i=1:Ns(2)
        for j =1:Ns(3)
            index = intersect(find(segmentLabel(:,1) == i), find(segmentLabel(:,2) == j));
            if (~isempty(index))
                step = t{1}(length(segments{1})+index);
                A = [A; [kron(Us{2}(i,:), Us{3}(j,:))]];
                B = [B; step];
            end
        end
    end
    X = A\B;
end

% Synthesize animations from all style combinations
numsteps = 200;
initY(1,1) = -30;
initY(2,1) = -20;
initY(3,1) = -10;
initY(4,1) = 0;
initY(5,1) = 10;
initY(6,1) = 20;
for i=1:Ns(2)
    for j =1:Ns(3)
        initY(2*i-1,1) = 0;
        if MODEL == 2
            step = [kron(Us{2}(i,:), Us{3}(j,:))]*X;

            X_star = [cos(linspace(0,numsteps*step,numsteps)') sin(linspace(0,numsteps*step,numsteps)')];
        else
            simStart = [Us{1}(segments{1}(2)+1,:) Us{1}(segments{1}(2),:)];
            % inputs 2 points in case using 2nd order model
            [X_star XRand_star] = ...
                simulatedynamics(Us{1}, segments{1}, Phps{1}, invKd, numsteps, simStart, priorModel{1});
        end

        Us_star{2} = Us{2}(i,:);
        Us_star{3} = Us{3}(j,:);

        for k = 1:numsteps
            Us_star{1} = X_star(k,:);
            % Us_star{1} = Us{1}(k,:);
            [Ysample(k,:) Ymean(k,:)] = ...
                sampleNModeReconstruction(Us_star, Us, Y, factor_idx, Hps, theta, beta, w, K, invK);
        end

        for d = 1:D
            Ysample(:,d) = Ysample(:,d)*sqrt(vY(d));
            Ymean(:,d) = Ymean(:,d)*sqrt(vY(d));
        end

        Data = Ymean+repmat(meanData, size(Ymean,1), 1);
        %initY(2*i-1,1) = j*10;
        saveMocapData( strcat('mgp_mean_',mat2str(i),'_',mat2str(j),'.amc'), Data ,initY(2*i-1,:));
    end
end
