addpath ../gpdm/src/netlab ../gpdm/src/gpdm ../gpdm/src/gplvm ...
    ../gpdm ../gpdm/src/util ./lbfgs ./LHSutils
dataDir = '../gpdm/';

format long;

global OPT_MASK L_BFGS;

% Define optimizer to use
L_BFGS = 1;

% Define the type of model to learn
% MODEL == 2 && INIT_CIRCLE == 1 means circular dyn. model from ICML 07'
% MODEL == 1 && INIT_CIRCLE == 0 m&& NOSTYLE == 1 means GPDM
MODEL = 1;
NOSTYLE = 1;

% How to initialize the content factor, set to one if MODEL == 2
INIT_CIRCLE = 0;

WVariance = 0;

load taylor_sample_data;
% [batchdata, seg] = selectsubset(batchdata, seg, 600);
% segments{1} = seg;
% Y = batchdata(1:585,:);
% Yt = batchdata(586:end,:);
% segments{1}(find(segments{1} > 585)) = [];

batchdata = batchdata([1:500, 701:705],:);
segments{1} = seg(1:2);
Y = batchdata(1:500,:); 
Yt = batchdata(501:end,:);

N = size(Y,1);
Nt = size(Yt,1);
D = size(Y,2);
factor_idx = [1:N]';

% Mean-substract the data, normalize the variance in each dimension
meanData = mean(batchdata);
Y = Y - repmat(meanData, N, 1);
Yt = Yt - repmat(meanData, Nt, 1);

vY = var(batchdata);
for d = 1:D
    Y(:,d) = Y(:,d)/sqrt(vY(d));
    Yt(:,d) = Yt(:,d)/sqrt(vY(d));
end
refYt = Yt;
%Yt(2:end,:) = nan;
FIX_ALPHA = 1;
if (MODEL == 1)
    % GPDM
    % priorModel{i}(1) : input of dynamics
    %   0 => [x_t, x_{t-1}]
    %   1 => [x_t, x_t - x_{t-1}]
    %   2 => [x_t]
    % priorModel{i}(2) : output of dynamics
    %   0 => x_{t+1}
    %   1 => x_{t+1} - x_t
    % priorModel{i}(3) : kernel type
    %   -1 => Isotropic Gaussian
    %   0 => RBF kernel with weighted dimensions, use with input 0 or 1
    %   1 => RBF kernel
    %   2 => Linear kernel
    %   3 => weighted Linear kernel + RBF kernel with weighted dimensions, use with
    %   input 0 or 1
    %   4 => weighted linear kernel
    %   5 => linear + RBF
    %
    % priorModel is only used if Phps is non-empty
    MGPopts{1} = struct('Ns', N, 'qs', 3, 'Nhps', 1, 'Nphps', 3, 'priorModel', [2 0 1], ...
        'periodic', 0, 'covS', [], 'meanS', [], 'G', [], 'BalanceTerm', 1);
    Hps{1} = 1;
    if (FIX_ALPHA == 1)
        Phps{1} = [0.01 0.2 1e6];
    else
        Phps{1} = [1 1 exp(1)];
    end
end

% Initialize hyperparameters
theta = 1;
beta = exp(1);

[v, u] = pca(Y);
v(find(v<0))=0;
Us{1} = Y*u(:, 1:MGPopts{1}.qs)*diag(1./sqrt(v(1:MGPopts{1}.qs)));
%plotseries(Us{1}, segments{1}, 'b');


w = ones(D,1);
extItr = 3;
inItr = 100;

trainL = [];
testL = [];


for iters = 1:extItr
    fprintf(2,'Iteration %d\n',iters);


    %         [K, invK] = ...
    %             computeNModeKernel(Us, Hps, factor_idx, theta, beta);
    %
    % Update scales every inItr iterations
    %         for d=1:D
    %             denom = Y(:,d)'*invK*Y(:,d);
    %             if (WVariance > 0)
    %                 denom = denom + 1/WVariance;
    %             end
    %             w(d) = sqrt(N/denom);
    %         end

    params = toOptimizerParams(Us, Hps, Phps, theta, beta);


    if (FIX_ALPHA == 1)
        OPT_MASK = length([Us{1}(:)' log(Hps{1})]) + [1:length([Phps{1}])];
    else
        OPT_MASK = [];
    end


    % Do not optimize the overall kernel scale, since we already have the
    % w, but comment this out if w is not optimized.
    %OPT_MASK = [OPT_MASK length(params)-1];

    optIdx = setdiff(1:length(params),OPT_MASK);
    realparams = params(optIdx);


    fn = @neglogprob_of_everything;
    if L_BFGS == 1
        %gradchek_lbfgs(params, fn);
        opts = lbfgs_options('iprint', 1, 'maxits', inItr, 'cb', @test_callback);
    else
        opts = optimset('Display','iter','GradObj','on','MaxIter', inItr, 'LargeScale', 'off', 'DerivativeCheck', 'on');
    end


    if (iters > 1)
        if L_BFGS == 1
            [realparams,fx,exitflag,userdata] = lbfgs(@(x)fn(x, params, Y, w, factor_idx, segments, MGPopts),realparams,opts);
            realparams = realparams';
        else
            [realparams,fx,exitflag] = fminunc(@(x)fn(x, params, Y, w, factor_idx, segments, MGPopts),realparams,opts);
        end
    end
    params(optIdx) = realparams;


    [Us, Hps, Phps, theta, beta] = fromOptimizerParams(params, MGPopts);
    
    teL = 0; 
    trL = 0; 
    for n = 2:size(Yt,1)
        [L1, trL, Ust, filledYt] = approximate_test_likelihood([Yt(n-1,:); nan(1,D)], toOptimizerParams(Us, Hps, Phps, theta, beta), ...
            Y, w, factor_idx, segments, MGPopts);
        L1 = sqrt(sum((Yt(n,:) - filledYt(end,:)).^2)); 
        teL = teL + L1; 
    end
    trainL = [trainL trL];
    testL = [testL teL/(size(Yt,1)-1)];


    figure(1);
    hold on;
    subplot(2,1,1);
    plot(trainL, 'b-o');
    subplot(2,1,2);
    plot(testL, 'r-o');
    hold off;

    [junk1, junk2, Ust, filledYt] = approximate_test_likelihood([Yt(1,:); nan(1,D)], toOptimizerParams(Us, Hps, Phps, theta, beta), ...
        Y, w, factor_idx, segments, MGPopts);
    
    allUst = Ust;
    filledY = filledYt;
    for i = 1:1000
        initUst{1} = [allUst{1}(end,:); 2*allUst{1}(end,:) - allUst{1}(end-1,:)]; 
%                    figure(2);
%         hold on;
%         plotseries(Us{1}, [1], 'b');
%         plotseries(allUst{1}, [1], 'r');
%          plotseries(initUst{1}, [1], 'g');
%         hold off;
%         pause;
        [junk1, junk2, Ust, filledYt] = approximate_test_likelihood([filledYt(end,:); nan(1,D)], toOptimizerParams(Us, Hps, Phps, theta, beta), ...
            Y, w, factor_idx, segments, MGPopts, initUst);
        allUst{1} = [allUst{1}; Ust{1}(end,:)];
        filledY = [filledY(1:end-1,:); filledYt];
    end
%     
            figure(2);
        hold on;
        plotseries(Us{1}, segments{1}, 'b');
      %  plotseries(X_pred, [1], 'r');
        plotseries(allUst{1}, [1], 'g');
        hold off;
    
%     [teL, trL, Ust, filledYt] = approximate_test_likelihood([Yt(1,:); nan(3,D); Yt(5,:)], toOptimizerParams(Us, Hps, Phps, theta, beta), ...
%         Y, w, factor_idx, segments, MGPopts);
    
end

return;



X = Us{1};
[Xin Xout] = priorIO(X, segments{1}, MGPopts{1}.priorModel);
[Kd invKd] = computePriorKernel(Xin, Phps{1}, MGPopts{1}.priorModel(3));
[K, invK] = ...
        computeNModeKernel(Us, Hps, factor_idx, theta, beta);
    
[X_pred XRand_pred] = ...
   simulatedynamics(X, segments{1}, Phps{1}, invKd, 1000, [zeros(1,3) allUst{1}(1,:)], MGPopts{1}.priorModel);
    

gpdmvisualise(X, Y, segments{1}, invK, [Hps{1} theta beta], invKd, Phps{1},  MGPopts{1}.priorModel, 0);

Ymean = [];
Ysample = [];
Yvar = []; 

for k = 1:1000
%     if (exist('Ust', 'var'))
%         Us_star{1} = Ust{1}(k,:);
%     else
%         Us_star{1} = Us{1}(k,:); 
%     end
    Us_star{1} = X_pred(k,:); 
    %Us_star{1} = Us{1}(k,:); 
    [Ysample(k,:) Ymean(k,:) Yvar(k)] = ...
        sampleNModeReconstruction(Us_star, Us, Y, factor_idx, Hps, theta, Beta, w, K, invK);
    
   k 
end

sum(sum((Ymean - Yt).^2))/size(Yt,1)

figure; 
Like = zeros(1,size(Yt,2)); 
for dof = 1:size(Yt,2)
    subplot(10,5,dof); hold on;
    plot(Yt(:,dof));
    plot(Ymean(:,dof),'m');
    plot(Ymean(:,dof) + sqrt(Yvar'),'r');
    plot(Ymean(:,dof) - sqrt(Yvar'),'r');
    axis([0 size(Yt,1) -2 2]);
    
    for k = 1:size(Yt,1)
        Like(dof) = Like(dof) -(0.5/Yvar(k))*(Yt(k,dof) - Ymean(k,dof))^2 - 0.5*log(Yvar(k)) - 0.5*(log(2*pi)) + log(w(dof)); 
    end
end

figure; 
hist(Like); 
for d = 1:D
    Ymean(:,d) = Ymean(:,d)*sqrt(vY(d));
    Ysample(:,d) = Ysample(:,d)*sqrt(vY(d));
end

Ymean = Ymean + repmat(meanData, size(Ymean,1), 1);
Ysample = Ysample + repmat(meanData, size(Ymean,1), 1); 

Ytt = refYt + repmat(meanData, size(refYt,1), 1);

for d = 1:D
    Ytr(:,d) = Y(:,d)*sqrt(vY(d));
    fY(:,d) = filledY(:,d)*sqrt(vY(d));
end
Ytr = Ytr + repmat(meanData, size(Y, 1),1);
fY = fY + repmat(meanData, size(filledY, 1), 1); 
showTaylorMotion(fY);

showTaylorMotion([Ytr]);
% Save learned model
save lastMGPModel Us Y factor_idx w Hps Phps theta beta N D meanData  ...
        segments MGPopts Yt;








